@@ -21,7 +21,9 @@ g_CORBA_Object_hash (gconstpointer key)
 	CORBA_Object obj = (gpointer) key;
 
 	retval = obj->type_qid;
-	g_slist_foreach (obj->profile_list, IOP_profile_hash, &retval);
+
+	g_assert (obj->object_key != NULL);
+	retval ^= IOP_ObjectKey_hash (obj->object_key);
 
 	return retval;
 }
@@ -263,7 +265,7 @@ ORBit_object_get_connection (CORBA_Object obj)
 					  &host, &service, &is_ssl, tbuf)) {
 
 			obj->connection = giop_connection_initiate (
-				proto, host, service,
+				obj->orb, proto, host, service,
 				is_ssl ? LINC_CONNECTION_SSL : 0, iiop_version);
 
 			if (ORBit_try_connection (obj)) {
@@ -481,6 +483,7 @@ CORBA_Object_is_a (CORBA_Object       obj,
 		   CORBA_Environment *ev)
 {
 	static GQuark  corba_object_quark = 0;
+	static GQuark  omg_corba_object_quark = 0;
 	CORBA_boolean  retval;
 	gpointer       args[] = { (gpointer *)&logical_type_id };
 	GQuark         logical_type_quark;
@@ -489,11 +492,18 @@ CORBA_Object_is_a (CORBA_Object       obj,
 		corba_object_quark = g_quark_from_static_string (
 			"IDL:CORBA/Object:1.0");
 
+	if (!omg_corba_object_quark)
+		omg_corba_object_quark = g_quark_from_static_string (
+			"IDL:omg.org/CORBA/Object:1.0");
+
 	logical_type_quark = g_quark_from_string (logical_type_id);
 
 	if (logical_type_quark == corba_object_quark)
 		return CORBA_TRUE;
 
+	if (logical_type_quark == omg_corba_object_quark)
+		return CORBA_TRUE;
+
 	if (!obj)
 		return CORBA_FALSE;
 
